@@ -85,7 +85,7 @@ class SafeMagic(object):
 
 def convert_page_path_to_nodes(text, node_list=[]):
     split = _RE_PAGE_PATH.split(text, maxsplit=1)
-    if len(split) == 7:
+    if len(split) == 8:
         pre = ''.join(split[:2])
         page_path = split[2]
         rest = ''.join(split[-2:])
@@ -95,10 +95,14 @@ def convert_page_path_to_nodes(text, node_list=[]):
     elif len(split) == 1:
         return node_list + [nodes.Text(text)]
 
+_PAGE_PATH_SE_SYMBOLS = ',!?;:(){}[]<>@#$%^&-+|\\~\'\"='
 _RE_PAGE_PATH = re.compile(
-    r'(?P<head>^|\s)'
-    r'(?P<page_path>(\.{0,2}/)+([a-zA-Z0-9][a-zA-Z0-9_\-\.\+]*/)*)'
-    r'(?P<tail>$|[\s\.!\?:;,])')
+    r'(?P<head>^|[\s%s])'
+    r'(?P<page_path>(/|(\.{1,2}/)+)([a-zA-Z0-9][a-zA-Z0-9_\-\.\+]*/)*)'
+    r'(?P<tail>$|[\s%s])'
+    % ('/' + re.escape(_PAGE_PATH_SE_SYMBOLS),  # can be starts with this
+       '.' + re.escape(_PAGE_PATH_SE_SYMBOLS),  # can be ends with this
+       ))
 
 
 def condition_page_path(nd):
@@ -499,6 +503,31 @@ def choice_from(*choices):
     return conv_func
 
 
+def check_rowdata_and_widths(rowdata, colwidths, warning):
+    """
+    Check if the number of column in `rowdata` and `widths` are the same
+    """
+    messages = []
+    if len(rowdata) > 0:
+        if colwidths is not None:
+            colwidthsstr = ' '.join(map(str, colwidths))
+            if len(rowdata[0]) > len(colwidths):
+                messages.append(warning(
+                    'Not enough widths arguments for table-data: '
+                    '"{0}"'.format(colwidthsstr)))
+                colwidths = None  # ignore the option
+            elif len(rowdata[0]) < len(colwidths):
+                messages.append(warning(
+                    'Too many widths arguments for table-data: '
+                    '"{0}"'.format(colwidthsstr)))
+                colwidths = None  # ignore the option
+        if len(rowdata[0]) < 1:
+            messages.append(warning('Not enough data'))
+    else:
+        messages.append(warning('Not enough data'))
+    return (messages, colwidths)
+
+
 class TableData(Directive):
     """
     Search data and show matched data and corresponding image(s)
@@ -507,6 +536,7 @@ class TableData(Directive):
     _dirc_name = 'table-data'
     _web = None  # needs override
     _DictTable = None  # needs override
+    _glob_list = None  # needs override
 
     required_arguments = 1
     optional_arguments = OPTIONAL_ARGUMENTS_INF
@@ -534,10 +564,11 @@ class TableData(Directive):
 
         base_syspath = path.join(datadir,
                                  self.options.get('base', ''))
-        data_syspath_list = glob_list([path.join(base_syspath,
-                                                 directives.uri(arg))
-                                       for arg in self.arguments],
-                                      glob_list_sorted)
+        data_syspath_list = self._glob_list(
+            [path.join(base_syspath,
+                       directives.uri(arg))
+             for arg in self.arguments],
+            glob_list_sorted)
         from_base_list = [
             path.relpath(x, base_syspath) for x in data_syspath_list]
 
@@ -566,12 +597,18 @@ class TableData(Directive):
                 ## link_magic.fails  # need to do something w/ fails
         if 'trans' in self.options:
             rowdata = zip(*rowdata)
+
+        (messages, colwidths) = check_rowdata_and_widths(
+            rowdata,
+            colwidths,
+            self.state.reporter.warning,
+            )
         return [gene_table(rowdata,
                            title=title_from_path(
                                self.arguments,
                                self.options.get('base'),
                                'Data found in: %s'),
-                           colwidths=colwidths)]
+                           colwidths=colwidths)] + messages
 
 
 class TableDataAndImage(Directive):
@@ -582,6 +619,7 @@ class TableDataAndImage(Directive):
     _dirc_name = 'table-data-and-image'
     _web = None  # needs override
     _DictTable = None  # needs override
+    _glob_list = None  # needs override
 
     required_arguments = 1
     optional_arguments = OPTIONAL_ARGUMENTS_INF
@@ -612,10 +650,11 @@ class TableDataAndImage(Directive):
 
         base_syspath = path.join(datadir,
                                  self.options.get('base', ''))
-        data_syspath_list = glob_list([path.join(base_syspath,
-                                                 directives.uri(arg))
-                                       for arg in self.arguments],
-                                      glob_list_sorted)
+        data_syspath_list = self._glob_list(
+            [path.join(base_syspath,
+                       directives.uri(arg))
+             for arg in self.arguments],
+            glob_list_sorted)
 
         data_keys = self.options.get('data', [])
         image_names = self.options.get('image', [])
@@ -650,12 +689,18 @@ class TableDataAndImage(Directive):
                             **image_options.get(i, {}))
                 for (i, name) in enumerate(image_names)]
             rowdata.append([col0] + images)
+
+        (messages, colwidths) = check_rowdata_and_widths(
+            rowdata,
+            colwidths,
+            self.state.reporter.warning,
+            )
         return [gene_table(rowdata,
                            title=title_from_path(
                                self.arguments,
                                self.options.get('base'),
                                'Data found in: %s'),
-                           colwidths=colwidths)]
+                           colwidths=colwidths)] + messages
 
 
 class DictDiff(Directive):
@@ -684,6 +729,7 @@ class FindImages(Directive):
     _dirc_name = 'find-images'
     _web = None  # needs override
     _DictTable = None  # needs override
+    _glob_list = None  # needs override
 
     required_arguments = 1
     optional_arguments = OPTIONAL_ARGUMENTS_INF
@@ -701,9 +747,10 @@ class FindImages(Directive):
 
         base_syspath = path.join(datadir,
                                  self.options.get('base', ''))
-        image_syspath_list = glob_list([path.join(base_syspath,
-                                                  directives.uri(arg))
-                                        for arg in self.arguments])
+        image_syspath_list = self._glob_list(
+            [path.join(base_syspath,
+                       directives.uri(arg))
+             for arg in self.arguments])
 
         def gene_image(relpath):
             image_node = nodes.image(
@@ -853,6 +900,70 @@ def setup_wiki(web=None, DictTable=None, glob_list=None):
         directives.register_directive(cls._dirc_name, cls)
 
 
+def safecall(tb_temp='%s'):
+    """
+    Decorator to call a function w/o a system error
+
+    This decorator adds two "hidden" arguments to the original function.
+
+    _debug : bool
+        If this argument is True, the error occurred int the function
+        call will not be suppressed.  (Default: False)
+    _mix : bool
+        If this argument is True, the traceback will be returned
+        instead of the original returned value when the error occurred.
+        If this argument is False, the original result and traceback
+        will be returned in 2-tuple always.  (Default: False)
+
+
+    >>> @safecall()
+    ... def somethingwrong(wrong):
+    ...     if wrong == 'wrong':
+    ...         raise Exception(wrong)
+    ...     return wrong
+    ...
+    >>> somethingwrong('nothing wrong')
+    'nothing wrong'
+    >>> somethingwrong('nothing wrong', _mix=False)
+    ('nothing wrong', None)
+    >>> tb = somethingwrong('wrong')  # error will be suppressed
+    >>> tb.splitlines()[-1]
+    'Exception: wrong'
+    >>> somethingwrong('wrong', _debug=True)  # error will be raised
+    Traceback (most recent call last):
+        ...
+    Exception: wrong
+
+    """
+
+    def decorator(func):
+        from functools import wraps
+
+        @wraps(func)
+        def wrapper(*args, **kwds):
+            mix = kwds.pop('_mix', True)
+            if kwds.pop('_debug', False):
+                result = func(*args, **kwds)
+                tb = None
+            else:
+                try:
+                    result = func(*args, **kwds)
+                    tb = None
+                except:
+                    import traceback
+                    import cgi
+                    result = None
+                    tb = tb_temp % cgi.escape(traceback.format_exc())
+            if mix:
+                return result if tb is None else tb
+            else:
+                return (result, tb)
+
+        return wrapper
+    return decorator
+
+
+@safecall('<h1>Failed to generate HTML</h1><pre>%s</pre>')
 def gene_html(text, page_path=None, settings_overrides={}):
     from docutils.core import publish_parts
     new_settings_overrides = SAFE_DOCUTILS.copy()
